#!/usr/bin/mpirun -np 2 python2.6-mpi

from numpy  import *
from math   import sqrt
from mcdemo import run_trials, mt19937p, sgenrand
from mpi4py import MPI

comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()


def run_mc(rtol = 1e-2, maxtrials = 1000000, nbatch = 500):
    """Run a parallel Monte Carlo experiment based on run_trials from mcdemo
    """
    mt = mt19937p()
    sgenrand(100+rank*10,mt)
    all_sum_X = 0
    all_sum_X2 = 0
    ntrials = 0
    errest = 2*rtol
    EX = 1
    local_results = zeros(2);
    total_results = zeros(2);
    while ntrials < maxtrials and errest > rtol*abs(EX):
        local_results[0], local_results[1] = run_trials(mt,nbatch)
        comm.Allreduce([local_results, MPI.DOUBLE], 
                       [total_results, MPI.DOUBLE], 
                       op=MPI.SUM)
        all_sum_X  = all_sum_X  + total_results[0]
        all_sum_X2 = all_sum_X2 + total_results[1]
        ntrials    = ntrials + nbatch*size
        EX   = all_sum_X /ntrials
        EX2  = all_sum_X2/ntrials
        stdX = sqrt(EX2-EX*EX)
        errest = stdX/sqrt(ntrials)
    return EX, errest


X, err = run_mc(1e-6)
if rank == 0:
    print X, err
